#
# Tempurity System Monthly Max Min Report Program
#
#
# This program is designed to be customized by our customers
#
# It was designed to run on Python 3.3 to 3.7.  You will need to download the appropriate Python to your computer.  
#
# This program produces a report in which a single max and min value is listed each day for every day of a month
# The time at which the maximum temperature/humidity/voltage etc, occurred is listed.
#
# Time Zone is an important input to this utility since all Tempurity Data is stored in Universal Time (UTC)
# Time Zone is extracted - but has to apply to the time zone to the data collection time.
#
# The program reads the raw data, calibration factor corrected data are not reported in this version, however the program can be customized
# to report corrected data. 
#
# This file should be in the Python directory, usually C:\Python33
# From the Windows command prompt "cd \"  then "cd Python33"
# 
# INPUTS
# In this program the month, and year, and approval line request are input by the user.  The list of devices to report is in a file called device-list.txt
# which is expected to also be in the Python directory
#
# The program could be invoked for April 2018 with no signature line from Python33 via the following
#
# c:\python33\python.exe minmax-reads-devicelist.py  04 2018 N
# 04 is the two digit month.  2018 is the 4 digit year  Y or N indicates whether a signature line is generated. 
#
# OUTPUTS
# The file e.g. "Tempurity Monthly MinMax Report-012F H-04-2018" is created with the report data - the Monitored Device name, the two digit month, and the 4 digit year are specified.
#

# There are different Tempurity System data directory locations depending on whether the Tempurity Server is running XP, WIN7/WIN8 and whether the server is 32 bit or 64 bit.
# This version is for 64 bit Win 7,8,10 computers
# Data directories for other OSs are included in the comments




import datetime
import time
import select
import struct
import sys
import subprocess
import telnetlib
import os
import zlib

print (sys.version)
print  (sys.argv[1])  # 3 digit device number doesn't error but doesn't print either



def remove(value):              # removes some of the illegal characters in Windows filenames
   value = value.replace('*', ' ')
   value = value.replace('/', ' ')
   
   value = value.replace(':', ' ')
   value = value.replace('?', ' ')
                         
   return value;

if __name__ == '__main__':

   debug = False

   # California time zone =8  - hours different from UK time 0

   # Timezone very important - must be run from the Timezone with the Tempurity Server
   # Otherwise this parameter should be changed manually
   # Because of Daylight Savings Time - the Timezone is different for different months
   
   timezone = time.altzone
   timezone = timezone/3600      
    
   
   dataunits = "undefined"

   print  (sys.argv[0])
   print ("\n")
   print ("Tempurity System Monthly Report \n")
   print (" ")


   devicenumber = ""    # just an initialization
   devicename = ""
   print (devicename)
   
   devicemonth = sys.argv[1]
   deviceyear = sys.argv[2]
   addsignature = sys.argv[3]

   if (devicenumber == ""):                      #Get the device number from device list file
      fileHandleC = open("device-list.txt",'r')
      
   field =0

   for dataline in fileHandleC:
      field = field + 1
      if (field == 1):
         devicename = dataline   
         devicename = devicename.strip()
         devicename = remove(devicename)
         
      if (field == 2):
         devicenumber = dataline  
         devicenumber = devicenumber.strip()
      if (field == 3):
         
         field = 0
      
      
      
         print (devicename)
         print (devicenumber)


         runmonth = time.strftime("%m", time.gmtime())
         imonth = int(runmonth)
         idevicemonth = int(devicemonth)
                     
                 
         if (((imonth >2) and (imonth <11)) and ((idevicemonth <3) or (idevicemonth>10))):
                  timezone = timezone +1

         if ((imonth <3) or (imonth >10)) and ((idevicemonth >2) and (idevicemonth<11)):
                  timezone = timezone -1

         #if (time.daylight):
         #         print ("The suggested timezone for the report month selected is", timezone)
         #         devicezone = input("You can override the default Timezone if you enter it here: ") 
         devicezone = ""  #delete if you reactivate the above
         
         # Converts Entered timezone to integer     
         if (devicezone == ""):
                  cnothing =0    # sorry coding gods
         else:
                  idevicezone = int(devicezone)
                  timezone = idevicezone
                  
         
         
         if (devicemonth=="01"):
                 charmonth = "January"
                 monthlimit=31
         if (devicemonth=="02"):
                 charmonth = "February"
                 monthlimit=28
         if (devicemonth=="03"):
                 charmonth = "March"
                 monthlimit=31
         if (devicemonth=="04"):
                 charmonth = "April"
                 monthlimit =30
         if (devicemonth=="05"):
                 charmonth = "May"
                 monthlimit=31
         if (devicemonth=="06"):
                 charmonth = "June"
                 monthlimit=30
         if (devicemonth=="07"):
                 charmonth = "July"
                 monthlimit=31
         if (devicemonth=="08"):
                 charmonth = "August"
                 monthlimit = 31
         if (devicemonth=="09"):
                 charmonth = "September"
                 monthlimit =30
         if (devicemonth=="10"):
                 charmonth = "October"
                 monthlimit = 31
         if (devicemonth=="11"):
                 charmonth = "November"
                 monthlimit = 30
         if (devicemonth=="12"):
                 charmonth = "December"
                 monthlimit = 31


         filenametowrite = "Tempurity Monthly MinMax Report" + "-" + devicename + "-" + devicemonth + "-" + deviceyear + ".txt"

       # Open the report file and write the report header 
         fileHandleW = open(filenametowrite,'w')
         fileHandleW.write("\n\n\nTempurity System Monthly Report of Automated Timepoint Values")
         fileHandleW.write("\n")
         fileHandleW.write("\n           for Monitored Device ")
         fileHandleW.write(devicenumber)
         fileHandleW.write("\n\n")
         fileHandleW.write("Device Name: " )
         fileHandleW.write(devicename)
         fileHandleW.write("        " )
         fileHandleW.write("Device Number: " )
         fileHandleW.write(devicenumber)
         fileHandleW.write("\n")
         fileHandleW.write("_____________________________________________________________________")
         fileHandleW.write ("\n\n")
         fileHandleW.write("Beginning Month Year: ")  
         #fileHandleW.write("Month: ")
         fileHandleW.write(charmonth)
         fileHandleW.write("     ")
         #fileHandleW.write("Year ")
         fileHandleW.write(deviceyear)
         fileHandleW.write("\n\n")
         #fileHandleW.write("Daily Reading Time:  ")
         #fileHandleW.write(str(reporthour))
         #fileHandleW.write(":00")
         fileHandleW.write("\n\n")
         fileHandleW.write("Local Timezone (UTC):  GMT -")
         fileHandleW.write(str(timezone))
         fileHandleW.write("\n")
         # uncomment the next line by deleting the number sign if you want the report run date listed
         #fileHandleW.write("Report run on: " + time.asctime(time.localtime()))
         


         gotTodaysData=0
         dayCtr = 0

         # All of the data for a single month is in the same directory
         # Figure out what that directory is
         # First the base directory:


         # when you uncomment these (take out the #) make sure that the indentation aligns with the other statments
         # Choose this for 32 bit XP Tempurity Servers
         #datadirectory1 = "C:\Windows\System32\Tempurity\Data" 

         # The below is for 64 bit XP machines
         #datadirectory1 = "C:\Windows\Syswow64\Tempurity\Data"
         
         # Choose the line below for WIN7 and 8 64 bit Tempurity Servers
         datadirectory1 = "C:" + "\\" + "Program Files (x86)" + "\\" +  "Networked Robotics" + "\\" + "Tempurity Server" + "\\" +"Tempurity" + "\\" + "Data"
         #C:\Program Files (x86)\Networked Robotics\Tempurity Server\Tempurity\Data

         # Choose the below for WIN7 and 8 32 bit Tempurity Servers
         #datadirectory1 = "C:" + "\\" + "Program Files" + "\\" +  "Networked Robotics" + "\\" + "Tempurity Server" + "\\" +"Tempurity
         # C:\Program Files\Networked Robotics\Tempurity Server\Tempurity\Data

         
         datadirectory2 = str(devicenumber) +"\\"+ str(deviceyear) + "\\" + str(devicemonth) + "\\"  
         datadirectory = datadirectory1 + "\\" + datadirectory2

            
         printedheader=0

         # Get 30 days of data starting with the first of the month
         while (dayCtr<monthlimit):

            dayCtr = dayCtr + 1
            # for deviceday = 1 to 30
            if (dayCtr<10):
               deviceday = "0" + str(dayCtr)
            else:
               deviceday = str(dayCtr)

            # Open the Tempurity System data file,  There is one file per monitored device per day
            # first get the file name
            # Filenames are going to be    
            # filename is ddd_yyyy_mm_dd.dat
            # an example filename is 035_2012_10_01.dat
            
            datafilename =str(devicenumber) + "_" + str(deviceyear) +  "_" + str(devicemonth) + "_" + deviceday + ".dat"

            datafilename = datadirectory + datafilename
            
            #print (datafilename)
            
            gotTodaysData=0
            

             
             # Convert Universal time to local time (  (u +(3600*tz))/86400) + 25569
             
             # open the Tempurity data file and search every record until 
         
               
            try:
               fileHandleR = open(datafilename,'r')
               tempmax = -5000.0
               tempmin = 5000.0
               
               for dataline in fileHandleR:
                  recctr =0
                 
                  nonstringvalue = 0
                  # read every line in the data file but if you already have a value don't do anything else
                  
                  if (gotTodaysData==0):
                     #dataline which holds a single line - time and value data
                     datatime = dataline [0:10]
                     datavalue = dataline [12:18]
                     float(datavalue)
                     nonstringvalue = datavalue
                     
                     float(nonstringvalue)
                     
                     #print (datavalue)
                     
                     if ( float(nonstringvalue) > tempmax ):
                         tempmax = float(nonstringvalue)
                         # calculate the printable time at which the max occurs
                         daysecs = int(datatime) - (timezone*3600)
                         daysecs = daysecs % 86400
                         temp = (daysecs%3600)/60
                         tempminutes = int((daysecs%3600)/60)
                         if (tempminutes < 10):
                               dayminutes = "0" + str(tempminutes)
                         else:
                               dayminutes = str(tempminutes)

                         temphour = int(daysecs/3600)
                         if ((daysecs/3600)<10):           
                               dayhour = "0" + str(temphour)
                         else:
                               dayhour = str(temphour)

                               dayhour = int(dayhour)
                               
                         #print (tempmax)
                     if ( float(nonstringvalue) < tempmin ):
                         tempmin = float(nonstringvalue)
                         #print (tempmin)    
                     dataunits = dataline [19:26]
                     #print (dataline)
                     endposition = dataunits.find("+")
                     
                     if (endposition != -1):
                        dataunits = dataunits [0: endposition-1]
                     
                     recctr = recctr + 1
                     

                     if (recctr ==1) and (printedheader==0):
                         printedheader=1
                         
                         fileHandleW.write("\n\n\n") 
                         fileHandleW.write("     Date   Local Time@Max   ")
                         if (dataunits=="Celsius"):
                                 parameter = "Temperature"
                         elif (dataunits=="%RH"):
                                 parameter = "Humidity"                     
                         else:
                                 parameter = "Value"
                                 
                         fileHandleW.write(parameter)
                         fileHandleW.write("(")
                         fileHandleW.write(dataunits)
                         
                         
                         fileHandleW.write (")")
                         fileHandleW.write("\n")

                         fileHandleW.write("                               Max Min ")
                         fileHandleW.write("\n")

                     # Tempurity records time as Universal Time so you need to know what time zone
                     # you are in Universal Time is the number of seconds since Jan 1 1970 
                     # and is based on GMT time zone
                     
                   
                         
               #  Print the max min line for this day/datafile
               fileHandleR.close()

               fileHandleW.write("\n")
               fileHandleW.write("        ")
               fileHandleW.write(deviceday)
               fileHandleW.write("     ")
               fileHandleW.write(str(dayhour))
               fileHandleW.write(":")
               fileHandleW.write(str(dayminutes))
               fileHandleW.write("          ")
               #fileHandleW.write(datavalue)

               fileHandleW.write(str(tempmax))
               fileHandleW.write(" ")
               fileHandleW.write(str(tempmin))




            
            except:
               # write a line that indicates an error for this day
               fileHandleW.write("\n")
               fileHandleW.write("        ")
               fileHandleW.write(deviceday)
               fileHandleW.write("     ")
               fileHandleW.write("No Data is Available ")
               #fileHandleW.write("\n")
              
            
            
             #  
             # Open up a notepad window with the report data
          
             # On Win XP 32 bit data directory is C:\Windows\System32\Tempurity\Data\'device id'\'year'\month\

         fileHandleW.write("\n\n")

         
         if ((addsignature == "Y") or (addsignature == "y")) :
            fileHandleW.write("\n\n\n\n")
            fileHandleW.write(" Reviewed by Signature ______________________________________")
            fileHandleW.write("\n\n\n\n")
            fileHandleW.write(" Released by Signature ______________________________________") 
            
         fileHandleW.close()

          # open a notepad window and print to show the results
         #r = os.popen("notepad /p Tempuritymonthreport.txt")

         # just open a notepad window

         notepadcommand = "notepad " + filenametowrite
         r = os.popen(notepadcommand)
         
         print ("\n\n")
      
   fileHandleC.close()
   sys.exit()

   


